package uk.co.sculptor.demos.stockcontrol;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * Created by Sculptor on 02/06/2015.
 *
 * @author Sculptor Ltd.
 */
public class Product implements Parcelable {
	private String m_ItemCode;
	private int m_StockLevel;
	private int m_ReorderLevel;
	private int m_EOQ;
	private int m_MOQ;
	private int m_CostPrice;
	private int m_SalePrice;
	private String m_UnitCode;
	private short m_LeadTime;
	private String m_NextDelDue;
	private String m_Description;
	/**
	 * This field is needed for Android to be able to create new objects, individually or as arrays.
	 */
	public static final Parcelable.Creator<Product> CREATOR = new Parcelable.Creator<Product>() {
		@Override
		public Product createFromParcel(Parcel in)
		{
			return new Product(in);
		}

		@Override
		public Product[] newArray(int size)
		{
			return new Product[size];
		}
	};

	/**
	 * @param src
	 *            The parcel to create the object from.
	 */
	private Product(Parcel src)
	{
		readFromParcel(src);
	}

	/**
	 * Constructor of Product that creates an empty Product
	 */
	public Product()
	{
	}

	/**
	 * Constructor of Product
	 */
	public Product(String ItemCode, int StockLevel, int ReorderLevel, int EOQ, int MOQ,
		int CostPrice, int SalePrice, String UnitCode, short LeadTime, String NextDelDue,
		String Description)
	{
		m_ItemCode = ItemCode;
		m_StockLevel = StockLevel;
		m_ReorderLevel = ReorderLevel;
		m_EOQ = EOQ;
		m_MOQ = MOQ;
		m_CostPrice = CostPrice;
		m_SalePrice = SalePrice;
		m_UnitCode = UnitCode;
		m_LeadTime = LeadTime;
		m_NextDelDue = NextDelDue;
		m_Description = Description;
	}

	/**
	 * Describe the kinds of special objects contained in this Parcelable's
	 * marshalled representation.
	 *
	 * @return a bitmask indicating the set of special object types marshalled
	 * by the Parcelable.
	 */
	@Override
	public int describeContents() {
		return 0;
	}

	/**
	 * Returns a string containing a concise, human-readable description of this
	 * object. Subclasses are encouraged to override this method and provide an
	 * implementation that takes into account the object's type and data. The
	 * default implementation is equivalent to the following expression:
	 * <pre>
	 *   getClass().getName() + '@' + Integer.toHexString(hashCode())</pre>
	 * <p>See <a href="{@docRoot}reference/java/lang/Object.html#writing_toString">Writing a useful
	 * {@code toString} method</a>
	 * if you intend implementing your own {@code toString} method.
	 *
	 * @return a printable representation of this object.
	 */
	@Override
	public String toString() {
		return m_Description;
	}

	/**
	 * Flatten this object in to a Parcel.
	 *
	 * @param dest  The Parcel in which the object should be written.
	 * @param flags Additional flags about how the object should be written.
	 *              May be 0 or {@link #PARCELABLE_WRITE_RETURN_VALUE}.
	 */
	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeString(getItemCode());
		dest.writeInt(getStockLevel());
		dest.writeInt(getReorderLevel());
		dest.writeInt(getEOQ());
		dest.writeInt(getMOQ());
		dest.writeInt(getCostPrice());
		dest.writeInt(getSalePrice());
		dest.writeString(getUnitCode());
		dest.writeInt(getLeadTime());
		dest.writeString(getNextDelDue());
		dest.writeString(getDescription());
	}

	/**
	 * Called from the constructor to create this object from a parcel.
	 *
	 * @param src
	 *            parcel from which to re-create object
	 */
	private void readFromParcel(Parcel src)
	{
		// We just need to read back each field in the order that it was written to the parcel
		setItemCode(src.readString());
		setStockLevel(src.readInt());
		setReorderLevel(src.readInt());
		setEOQ(src.readInt());
		setMOQ(src.readInt());
		setCostPrice(src.readInt());
		setSalePrice(src.readInt());
		setUnitCode(src.readString());
		setLeadTime((short)src.readInt());
		setNextDelDue(src.readString());
		setDescription(src.readString());
	}

	public String getItemCode() {
		return m_ItemCode;
	}

	public void setItemCode(String itemCode) {
		m_ItemCode = itemCode;
	}

	public int getStockLevel() {
		return m_StockLevel;
	}

	public void setStockLevel(int stockLevel) {
		m_StockLevel = stockLevel;
	}

	public int getReorderLevel() {
		return m_ReorderLevel;
	}

	public void setReorderLevel(int reorderLevel) {
		m_ReorderLevel = reorderLevel;
	}

	public int getEOQ() {
		return m_EOQ;
	}

	public void setEOQ(int EOQ) {
		m_EOQ = EOQ;
	}

	public int getMOQ() {
		return m_MOQ;
	}

	public void setMOQ(int MOQ) {
		m_MOQ = MOQ;
	}

	public int getCostPrice() {
		return m_CostPrice;
	}

	public void setCostPrice(int costPrice) {
		m_CostPrice = costPrice;
	}

	public int getSalePrice() {
		return m_SalePrice;
	}

	public void setSalePrice(int salePrice) {
		m_SalePrice = salePrice;
	}

	public String getUnitCode() {
		return m_UnitCode;
	}

	public void setUnitCode(String unitCode) {
		m_UnitCode = unitCode;
	}

	public short getLeadTime() {
		return m_LeadTime;
	}

	public void setLeadTime(short leadTime) {
		m_LeadTime = leadTime;
	}

	public String getNextDelDue() {
		return m_NextDelDue;
	}

	public void setNextDelDue(String nextDelDue) {
		m_NextDelDue = nextDelDue;
	}

	public String getDescription() {
		return m_Description;
	}

	public void setDescription(String description) {
		m_Description = description;
	}
}
