package uk.co.sculptor.demos.stockcontrol;

import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

/**
 * A fragment representing a single Product detail screen.
 * This fragment is either contained in a {@link ProductListActivity}
 * in two-pane mode (on tablets) or a {@link ProductDetailActivity}
 * on handsets.
 */
public class ProductDetailFragment extends Fragment {
    /**
     * The fragment argument representing the item ID that this fragment
     * represents.
     */
    public static final String ARG_PRODUCT_ID = "product_id";

    /**
     * The dummy content this fragment is presenting.
     */
    private Product m_product;

    /**
     * Mandatory empty constructor for the fragment manager to instantiate the
     * fragment (e.g. upon screen orientation changes).
     */
    public ProductDetailFragment() {
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        if (getArguments().containsKey(ARG_PRODUCT_ID)) {
            // Load the dummy content specified by the fragment
            // arguments. In a real-world scenario, use a Loader
            // to load content from a content provider.
            m_product = StockControl.getProductById(getArguments().getString(ARG_PRODUCT_ID));
        }
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_product_detail, container, false);

        // Show the dummy content as text in a TextView.
        if (m_product == null) {
	        return rootView;
        }

	    ((TextView)rootView.findViewById(R.id.item_code)).setText("Item code: " + m_product.getItemCode());
		((TextView)rootView.findViewById(R.id.product_detail)).setText("Description: " + m_product.getDescription());
	    ((TextView)rootView.findViewById(R.id.stock_level)).setText("Stock level: " + Integer.toString(m_product.getStockLevel()));
	    ((TextView)rootView.findViewById(R.id.reorder_level)).setText("Reorder level: " + Integer.toString(m_product.getReorderLevel()));
	    ((TextView)rootView.findViewById(R.id.eoq)).setText("EOQ: " + Integer.toString(m_product.getEOQ()));
	    ((TextView)rootView.findViewById(R.id.moq)).setText("MOQ: " + Integer.toString(m_product.getMOQ()));
	    ((TextView)rootView.findViewById(R.id.cost_price)).setText("Cost price: " + Integer.toString(m_product.getCostPrice()));
	    ((TextView)rootView.findViewById(R.id.sale_price)).setText("Sale price: " + Integer.toString(m_product.getSalePrice()));
	    ((TextView)rootView.findViewById(R.id.unit_code)).setText("Unit code: " + m_product.getUnitCode());
	    ((TextView)rootView.findViewById(R.id.lead_time)).setText("Lead time: " + Integer.toString(m_product.getLeadTime()));
	    ((TextView)rootView.findViewById(R.id.next_del_due)).setText("Due: " + m_product.getNextDelDue());

        return rootView;
    }
}
