import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.EnumSet;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import uk.co.sculptor.kflibjava.Database;
import uk.co.sculptor.kflibjava.Field;
import uk.co.sculptor.kflibjava.FileIndex;
import uk.co.sculptor.kflibjava.KeyedFile;
import uk.co.sculptor.kflibjava.KeyedFile.Mode;
import uk.co.sculptor.kflibjava.KfException;
import uk.co.sculptor.kflibjava.Realm;
import uk.co.sculptor.kflibjava.Server;

/**
 * Copyright Sculptor Limited 1997-2015
 */

/**
 * kflibjava example test application.
 * 
 * @author Marcos Gutirrez Batz
 */
public class StockControl extends JFrame
{

	/**
	 * Enumerator used to define the possible operations that may use the Ok/Cancel buttons
	 * 
	 * @author Marcos Gutirrez Batz
	 */
	private enum Operation
	{
		NONE, AMEND, INSERT, FIND
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Launch the application.
	 * 
	 * @param args
	 *            CLI arguments
	 */
	public static void main(String[] args)
	{
		EventQueue.invokeLater(new Runnable() {
			@Override
			public void run()
			{
				try {
					StockControl stockControl = new StockControl();
					stockControl.initData();
					stockControl.setVisible(true);
				}
				catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	private JPanel m_contentPane;

	private JTextField m_itemCodeField;
	private JTextField m_stockLevelField;
	private JTextField m_reorderLevelField;
	private JTextField m_EOQField;
	private JTextField m_MOQField;
	private JTextField m_costField;
	private JTextField m_saleField;
	private JTextField m_unitCodeField;
	private JTextField m_leadTimeField;
	private JTextField m_nextDeliveryField;
	private JTextField m_descriptionField;
	private JComboBox<String> m_indexBox;

	private Realm m_realm;

	private Server m_server;

	private Database m_database;

	private KeyedFile m_stock;

	private Field m_itemCode;

	private Field m_stockLevel;

	private Field m_reorderLevel;

	private Field m_EOQ;

	private Field m_MOQ;

	private Field m_costPrice;

	private Field m_salePrice;

	private Field m_unitCode;

	private Field m_leadTime;

	private Field m_nextDelDue;

	private Field m_description;

	private FileIndex m_nextDeliveryIndex;

	private Operation m_operation;
	private JButton m_btnOk;
	private JButton m_btnCancel;
	private JButton m_btnFind;
	private JButton m_btnInsert;
	private JButton m_btnNext;
	private JButton m_btnAmend;
	private JButton m_btnPrev;
	private JButton m_btnDelete;
	private JButton m_btnMatch;
	private JButton m_btnRewind;
	private JButton m_btnWind;
	private JButton m_btnExit;

	/**
	 * Create the frame.
	 */
	public StockControl()
	{
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e)
			{
				endData();
			}
		});
		setResizable(false);
		setTitle("Stock Control kflibjava test");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 539, 486);
		m_contentPane = new JPanel();
		m_contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(m_contentPane);
		m_contentPane.setLayout(null);

		JLabel lblIndex = new JLabel("Index");
		lblIndex.setHorizontalAlignment(SwingConstants.TRAILING);
		lblIndex.setBounds(381, 27, 49, 14);
		m_contentPane.add(lblIndex);

		m_indexBox = new JComboBox<String>();
		m_indexBox.setBounds(438, 24, 85, 20);
		m_contentPane.add(m_indexBox);
		m_indexBox.setModel(new DefaultComboBoxModel<String>(new String[] { "Main", "Next delivery" }));
		m_indexBox.setSelectedIndex(0);
		lblIndex.setLabelFor(m_indexBox);

		JLabel lblItemCode = new JLabel("Item Code");
		lblItemCode.setBounds(10, 42, 99, 14);
		m_contentPane.add(lblItemCode);
		lblItemCode.setLabelFor(m_itemCodeField);
		lblItemCode.setHorizontalAlignment(SwingConstants.TRAILING);

		m_itemCodeField = new JTextField();
		m_itemCodeField.setEditable(false);
		m_itemCodeField.setBounds(119, 42, 257, 20);
		m_contentPane.add(m_itemCodeField);
		m_itemCodeField.setColumns(10);

		JLabel lblStockLevel = new JLabel("Stock Level");
		lblStockLevel.setBounds(10, 73, 99, 14);
		m_contentPane.add(lblStockLevel);
		lblStockLevel.setLabelFor(m_stockLevelField);
		lblStockLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_stockLevelField = new JTextField();
		m_stockLevelField.setEditable(false);
		m_stockLevelField.setBounds(119, 72, 257, 20);
		m_contentPane.add(m_stockLevelField);
		m_stockLevelField.setColumns(10);

		JLabel lblReorderLevel = new JLabel("Reorder Level");
		lblReorderLevel.setBounds(10, 104, 99, 14);
		m_contentPane.add(lblReorderLevel);
		lblReorderLevel.setHorizontalAlignment(SwingConstants.TRAILING);
		lblReorderLevel.setLabelFor(m_reorderLevelField);

		m_reorderLevelField = new JTextField();
		m_reorderLevelField.setEditable(false);
		m_reorderLevelField.setBounds(119, 104, 257, 20);
		m_contentPane.add(m_reorderLevelField);
		m_reorderLevelField.setColumns(10);

		JLabel lblEOQLevel = new JLabel("EOQ");
		lblEOQLevel.setBounds(10, 140, 99, 14);
		m_contentPane.add(lblEOQLevel);
		lblEOQLevel.setLabelFor(m_EOQField);
		lblEOQLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_EOQField = new JTextField();
		m_EOQField.setEditable(false);
		m_EOQField.setBounds(119, 141, 257, 20);
		m_contentPane.add(m_EOQField);
		m_EOQField.setColumns(10);

		JLabel lblMOQLevel = new JLabel("MOQ");
		lblMOQLevel.setBounds(10, 176, 99, 14);
		m_contentPane.add(lblMOQLevel);
		lblMOQLevel.setLabelFor(m_MOQField);
		lblMOQLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_MOQField = new JTextField();
		m_MOQField.setEditable(false);
		m_MOQField.setBounds(119, 176, 257, 20);
		m_contentPane.add(m_MOQField);
		m_MOQField.setColumns(10);

		JLabel lblCostLevel = new JLabel("Cost Price");
		lblCostLevel.setBounds(10, 208, 99, 14);
		m_contentPane.add(lblCostLevel);
		lblCostLevel.setLabelFor(m_costField);
		lblCostLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_costField = new JTextField();
		m_costField.setEditable(false);
		m_costField.setBounds(119, 208, 257, 20);
		m_contentPane.add(m_costField);
		m_costField.setColumns(10);

		JLabel lblSaleLevel = new JLabel("Sale Price");
		lblSaleLevel.setBounds(10, 243, 99, 14);
		m_contentPane.add(lblSaleLevel);
		lblSaleLevel.setLabelFor(m_saleField);
		lblSaleLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_saleField = new JTextField();
		m_saleField.setEditable(false);
		m_saleField.setBounds(119, 244, 257, 20);
		m_contentPane.add(m_saleField);
		m_saleField.setColumns(10);

		JLabel lblUnitCodeLevel = new JLabel("Unit Code");
		lblUnitCodeLevel.setBounds(10, 279, 99, 14);
		m_contentPane.add(lblUnitCodeLevel);
		lblUnitCodeLevel.setHorizontalAlignment(SwingConstants.TRAILING);
		lblUnitCodeLevel.setLabelFor(m_unitCodeField);

		m_unitCodeField = new JTextField();
		m_unitCodeField.setEditable(false);
		m_unitCodeField.setBounds(119, 282, 257, 20);
		m_contentPane.add(m_unitCodeField);
		m_unitCodeField.setColumns(10);

		JLabel lblLeadTimeLevel = new JLabel("Lead Time");
		lblLeadTimeLevel.setBounds(10, 321, 99, 14);
		m_contentPane.add(lblLeadTimeLevel);
		lblLeadTimeLevel.setLabelFor(m_leadTimeField);
		lblLeadTimeLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_leadTimeField = new JTextField();
		m_leadTimeField.setEditable(false);
		m_leadTimeField.setBounds(119, 319, 257, 20);
		m_contentPane.add(m_leadTimeField);
		m_leadTimeField.setColumns(10);

		JLabel lblNextDeliveryLevel = new JLabel("Next Delivery");
		lblNextDeliveryLevel.setBounds(10, 355, 99, 14);
		m_contentPane.add(lblNextDeliveryLevel);
		lblNextDeliveryLevel.setHorizontalAlignment(SwingConstants.TRAILING);
		lblNextDeliveryLevel.setLabelFor(m_nextDeliveryField);

		m_nextDeliveryField = new JTextField();
		m_nextDeliveryField.setEditable(false);
		m_nextDeliveryField.setBounds(119, 356, 257, 20);
		m_contentPane.add(m_nextDeliveryField);
		m_nextDeliveryField.setColumns(10);

		JLabel lblDescriptionLevel = new JLabel("Description");
		lblDescriptionLevel.setBounds(10, 389, 99, 14);
		m_contentPane.add(lblDescriptionLevel);
		lblDescriptionLevel.setLabelFor(m_descriptionField);
		lblDescriptionLevel.setHorizontalAlignment(SwingConstants.TRAILING);

		m_descriptionField = new JTextField();
		m_descriptionField.setEditable(false);
		m_descriptionField.setBounds(119, 389, 257, 20);
		m_contentPane.add(m_descriptionField);
		m_descriptionField.setColumns(10);

		m_btnFind = new JButton("Find");
		m_btnFind.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionFindButton();
			}
		});
		m_btnFind.setBounds(438, 60, 85, 23);
		m_contentPane.add(m_btnFind);

		m_btnInsert = new JButton("Insert");
		m_btnInsert.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionInsertButton();
			}
		});
		m_btnInsert.setBounds(438, 99, 85, 23);
		m_contentPane.add(m_btnInsert);

		m_btnNext = new JButton("Next");
		m_btnNext.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev)
			{
				actionNextButton();
			}
		});
		m_btnNext.setBounds(438, 138, 85, 23);
		m_contentPane.add(m_btnNext);

		m_btnAmend = new JButton("Amend");
		m_btnAmend.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionAmendButton();
			}
		});
		m_btnAmend.setBounds(438, 177, 85, 23);
		m_contentPane.add(m_btnAmend);

		m_btnPrev = new JButton("Prev");
		m_btnPrev.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionPrevButton();
			}
		});
		m_btnPrev.setBounds(438, 216, 85, 23);
		m_contentPane.add(m_btnPrev);

		m_btnDelete = new JButton("Delete");
		m_btnDelete.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionDeleteButton();
			}
		});
		m_btnDelete.setBounds(438, 255, 85, 23);
		m_contentPane.add(m_btnDelete);

		m_btnMatch = new JButton("Match");
		m_btnMatch.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionMatchButton();
			}
		});
		m_btnMatch.setBounds(438, 294, 85, 23);
		m_contentPane.add(m_btnMatch);

		m_btnRewind = new JButton("Rewind");
		m_btnRewind.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionRewindButton();
			}
		});
		m_btnRewind.setBounds(438, 333, 85, 23);
		m_contentPane.add(m_btnRewind);

		m_btnWind = new JButton("Wind");
		m_btnWind.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionWindButton();
			}
		});
		m_btnWind.setBounds(438, 372, 85, 23);
		m_contentPane.add(m_btnWind);

		m_btnExit = new JButton("Exit");
		m_btnExit.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				endData();
				dispose();
			}
		});
		m_btnExit.setBounds(438, 411, 85, 23);
		m_contentPane.add(m_btnExit);

		m_btnOk = new JButton("Ok");
		m_btnOk.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionOkButton();
			}
		});
		m_btnOk.setVisible(false);
		m_btnOk.setBounds(119, 424, 89, 23);
		m_contentPane.add(m_btnOk);

		m_btnCancel = new JButton("Cancel");
		m_btnCancel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e)
			{
				actionCancelButton();
			}
		});
		m_btnCancel.setVisible(false);
		m_btnCancel.setBounds(287, 424, 89, 23);
		m_contentPane.add(m_btnCancel);
	}

	/**
	 * Method to start an amend operation.
	 */
	private void actionAmendButton()
	{
		/* Let's check if there is a record selected */
		if (!m_stock.check()) {
			JOptionPane.showMessageDialog(null, "No Record Selected");
			return;
		}

		/* Save the AMEND operation */
		m_operation = Operation.AMEND;

		/* Make the text fields editable */
		setFieldsEditable(true);

		/* Show the OK/Cancel buttons */
		setOkCancelVisible(true);
	}

	/**
	 * Method to cancel an amend, find or insert operation when pressing the cancel button.
	 */
	private void actionCancelButton()
	{
		/* Make the text fields non-editable */
		setFieldsEditable(false);

		/* Hide the OK/Cancel buttons */
		setOkCancelVisible(false);

		/* Lets display the previous values */
		txtfldRecordToDisplay();

		/* Remove the current operation */
		m_operation = Operation.NONE;
	}

	/**
	 * Method to delete current record.
	 */
	private void actionDeleteButton()
	{
		/* Let's check if there is a record selected */
		if (!m_stock.check()) {
			JOptionPane.showMessageDialog(null, "No Record Selected");
			return;
		}

		try {
			m_stock.deleteRecord();
			txtfldClearDisplay();
		}
		catch (IOException e) {
			e.printStackTrace();
			endData();
			System.exit(1);
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Method to start a find operation.
	 */
	private void actionFindButton()
	{
		/* Clean the text fields */
		txtfldClearDisplay();

		/* Save the AMEND operation */
		m_operation = Operation.FIND;

		/* Make the text fields editable */
		setFieldsEditable(true);

		/* Show the OK/Cancel buttons */
		setOkCancelVisible(true);
	}

	/**
	 * Method to start an insert operation.
	 */
	private void actionInsertButton()
	{
		/* Clean the text fields */
		txtfldClearDisplay();

		/* Save the AMEND operation */
		m_operation = Operation.INSERT;

		/* Make the text fields editable */
		setFieldsEditable(true);

		/* Show the OK/Cancel buttons */
		setOkCancelVisible(true);
	}

	/**
	 * Advance to next matched record.
	 */
	private void actionMatchButton()
	{
		try {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryIndex.match();
			else
				m_stock.match();

			txtfldRecordToDisplay();
		}
		catch (IOException e) {
			e.printStackTrace();
			endData();
			System.exit(1);
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Advance to next record.
	 */
	private void actionNextButton()
	{
		try {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryIndex.next();
			else
				m_stock.next();

			txtfldRecordToDisplay();
		}
		catch (IOException e) {
			e.printStackTrace();
			endData();
			System.exit(1);
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Method to start an amend, find or insert operation when pressing the OK button.
	 */
	private void actionOkButton()
	{
		/* Make the text fields non-editable */
		setFieldsEditable(false);

		/* Hide the OK/Cancel buttons */
		setOkCancelVisible(false);

		if (m_operation == Operation.NONE)
			return;

		/* Copy the text field strings to each field */
		if (!txtfldDisplayToRecord()) {
			/* Conversion/formating problem lets force the user to fix or cancel the edit */
			setOkCancelVisible(true);
			setFieldsEditable(true);
			return;
		}

		try {
			/* Lets have a look at the operation initiated */
			switch (m_operation) {
			case AMEND:
				m_stock.writel();
				break;
			case FIND:
				if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
					m_nextDeliveryIndex.find();
				else
					m_stock.find();
				break;
			case INSERT:
				m_stock.insert();
				break;
			default:
				break;
			}
		}
		catch (IOException e) {
			e.printStackTrace();
			endData();
			System.exit(1);
		}
		catch (KfException e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, e.getMessage());
			/* Error. So continue with the edition */
			setOkCancelVisible(true);
			setFieldsEditable(true);
			return;
		}

		/* Operation finished correctly, lets display the results */
		txtfldRecordToDisplay();

		/* Remove the current operation */
		m_operation = Operation.NONE;
	}

	/**
	 * Advance to previous record.
	 */
	private void actionPrevButton()
	{
		try {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryIndex.prev();
			else
				m_stock.prev();

			txtfldRecordToDisplay();
		}
		catch (IOException e) {
			e.printStackTrace();
			endData();
			System.exit(1);
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Rewind current index.
	 */
	private void actionRewindButton()
	{
		try {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryIndex.rewind();
			else
				m_stock.rewind();

			txtfldClearDisplay();
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Wind current index.
	 */
	private void actionWindButton()
	{
		try {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryIndex.wind();
			else
				m_stock.wind();

			txtfldClearDisplay();
		}
		catch (KfException e) {
			/* Issue the message */
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Method to specify if the text fields are editable.
	 * 
	 * @param editable
	 *            true when the text fields may be modified.
	 */
	private void setFieldsEditable(boolean editable)
	{
		if (m_operation == Operation.FIND) {
			if (m_indexBox.getSelectedIndex() == 1) /* == 1 means Next delivery date index */
				m_nextDeliveryField.setEditable(editable);
			else
				m_itemCodeField.setEditable(editable);
		}
		else {
			/* Display all the fields converted to string */
			m_itemCodeField.setEditable(editable);
			m_stockLevelField.setEditable(editable);
			m_reorderLevelField.setEditable(editable);
			m_EOQField.setEditable(editable);
			m_MOQField.setEditable(editable);
			m_costField.setEditable(editable);
			m_saleField.setEditable(editable);
			m_unitCodeField.setEditable(editable);
			m_leadTimeField.setEditable(editable);
			m_nextDeliveryField.setEditable(editable);
			m_descriptionField.setEditable(editable);
		}
	}

	/**
	 * @param visible
	 *            true when the buttons must be visible, false otherwise.
	 */
	private void setOkCancelVisible(boolean visible)
	{
		m_btnCancel.setVisible(visible);
		m_btnOk.setVisible(visible);

		m_btnFind.setVisible(!visible);
		m_btnInsert.setVisible(!visible);
		m_btnNext.setVisible(!visible);
		m_btnAmend.setVisible(!visible);
		m_btnPrev.setVisible(!visible);
		m_btnDelete.setVisible(!visible);
		m_btnMatch.setVisible(!visible);
		m_btnRewind.setVisible(!visible);
		m_btnWind.setVisible(!visible);
		m_btnExit.setVisible(!visible);

	}

	/**
	 * Clean the UI text fields
	 */
	private void txtfldClearDisplay()
	{
		m_itemCodeField.setText("");
		m_stockLevelField.setText("");
		m_reorderLevelField.setText("");
		m_EOQField.setText("");
		m_MOQField.setText("");
		m_costField.setText("");
		m_saleField.setText("");
		m_unitCodeField.setText("");
		m_leadTimeField.setText("");
		m_nextDeliveryField.setText("");
		m_descriptionField.setText("");
	}

	/**
	 * @return true when all the conversions work correctly.
	 */
	private boolean txtfldDisplayToRecord()
	{
		/* Set the displayed data on the fields */
		try {
			m_itemCode.setString(m_itemCodeField.getText());
			m_stockLevel.setString(m_stockLevelField.getText());
			m_reorderLevel.setString(m_reorderLevelField.getText());
			m_EOQ.setString(m_EOQField.getText());
			m_MOQ.setString(m_MOQField.getText());
			m_costPrice.setString(m_costField.getText());
			m_salePrice.setString(m_saleField.getText());
			m_unitCode.setString(m_unitCodeField.getText());
			m_leadTime.setString(m_leadTimeField.getText());
			m_nextDelDue.setString(m_nextDeliveryField.getText());
			m_description.setString(m_descriptionField.getText());
		}
		catch (KfException e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
			return false;
		}

		return true;
	}

	/**
	 * Display the data from the sculptor file on the textboxes.
	 */
	private void txtfldRecordToDisplay()
	{
		try {
			/* Display all the fields converted to string */
			m_itemCodeField.setText(m_itemCode.getString());
			m_stockLevelField.setText(m_stockLevel.getString());
			m_reorderLevelField.setText(m_reorderLevel.getString());
			m_EOQField.setText(m_EOQ.getString());
			m_MOQField.setText(m_MOQ.getString());
			m_costField.setText(m_costPrice.getString());
			m_saleField.setText(m_salePrice.getString());
			m_unitCodeField.setText(m_unitCode.getString());
			m_leadTimeField.setText(m_leadTime.getString());
			m_nextDeliveryField.setText(m_nextDelDue.getString());
			m_descriptionField.setText(m_description.getString());
		}
		catch (KfException e) {
			JOptionPane.showMessageDialog(null, e.getMessage());
			e.printStackTrace();
		}
	}

	/**
	 * Close the Sculptor objects.
	 */
	protected void endData()
	{
		/* Closing the realm all the server, databases, keyed files, indexes and fields are going to be closed */
		try {
			if (m_realm != null) {
				m_realm.close();
				m_realm = null;
			}
		}
		catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Called from the Garbage collector to free resources.
	 * 
	 * @see java.lang.Object#finalize()
	 */
	@Override
	protected void finalize() throws Throwable
	{
		try {
			/* Call the StockControl endData method */
			endData();
		}
		finally {
			super.finalize();
		}
	}

	/**
	 * Initialise the Sculptor needed objects.
	 * 
	 * @throws IOException
	 * @throws KfException
	 * @throws InterruptedException
	 */
	protected void initData() throws KfException, IOException, InterruptedException
	{
		/*
		 * Create the realm. Is an object that contains related servers from the transaction point of view.
		 * In this case is not needed as we are not going to use transactions on this example
		 */
		m_realm = new Realm("StockControlRealm");

		/* Open a server connection. Server must be running on localhost with a 10 second time out for the
		 * connection
		 */
		m_server = m_realm.openServer("localhost", 10);

		/* Open a database, which in this case specifies the source directory where the KeyedFile's are
		 * relative to
		 */
		m_database = m_server.openDB("StockControlDatabase", Database.Type.SCULPTOR,
				EnumSet.noneOf(Database.Flag.class), "C:/sc/demos/stockcontrol/data", 5,
				"C:/sc/demos/stockcontrol/logs/stockcontrol.log");

		/* Open the stock file */
		m_stock = m_database.openTable("stock", Mode.UPDATE);

		/* Get the fields and indexes to work with */
		m_itemCode = m_stock.getField("ItemCode");
		m_stockLevel = m_stock.getField("StockLevel");
		m_reorderLevel = m_stock.getField("ReorderLevel");
		m_EOQ = m_stock.getField("EOQ");
		m_MOQ = m_stock.getField("MOQ");
		m_costPrice = m_stock.getField("CostPrice");
		m_salePrice = m_stock.getField("SalePrice");
		m_unitCode = m_stock.getField("UnitCode");
		m_leadTime = m_stock.getField("LeadTime");
		m_nextDelDue = m_stock.getField("NextDelDue");
		m_description = m_stock.getField("Description");

		m_nextDeliveryIndex = m_stock.getIndex("next_del_index");
	}
}
