/* stock.r */

/********************************************************************************/

!include <sculptor.h>
!include <printer.h>

/********************************************************************************/

!define _FIND 1
!define _INSERT 2
!define _AMEND 3

!define _LOCK 1
!define _UNLOCK 2

/********************************************************************************/

!file FILE_STOCK "../data/stock"

!record TMP_STOCK = FILE_STOCK
!record LastRec = FILE_STOCK

/********************************************************************************/

!record DispListTable {
	ItemCode,"Item Code",a20
	StockLevel,"Stock Level",i4,"#,###,##0"
	ReorderLevel,"Reorder Level",i4,"#,###,##0"
	EOQ,"EOQ",i4,"#,###,##0"
	MOQ,"MOQ",i4,"#,###,##0"
	CostPrice,"Cost Price",n4.0/my,"###,###.##"
	SalePrice,"Sale Price",n4.0/my,"###,###.##"
	UnitCode,"Unit Code",a4
	LeadTime,"Lead Time",i2,"####"
	NextDelDue,"Next Delivery",i4/dn,"dD/mM/YYYY"
	Description,"Description",a60
}

!record ListTableCellProps {
	cell_type,,i4[11]
	cell_width,,i2[11]
}


/********************************************************************************/

!temp CurrentIndex,,a32
!temp NumSelIndexes,,i2
!temp LastFindIndex,,a32
!temp CurrentIndexNum,,i2
!temp ContinueFind,,i2
!temp Selected,,i2
!temp Inserted,,i2
!temp CurrentOpt,,a20
!temp Found,,i2
!temp LockMain,,i2

/********************************************************************************/

+window wintask at 480, 237 {
	max_width = 640
	max_height = 385
	style = WS_CLOSEBUTTON | WS_MINBUTTON | WS_CAPTION
	event = WinTaskEvent
	title = "stock"
	data_font = "arial@8"
	label_font = "arial@8"

	+text IndexListText at 435, 15 {
		 label = "Index"
	}

	+listbox IndexList at 470, 10 {
		listbox_type = LIST_BUTTON
		width = 150
		rows = 2
		event = IndexListEvent
	}

	button group OptButs {

		+button FindBut at 490, 55 {
			label = "&Find"
			width = 60
			event = FindButEvent
			tooltip = "Find a record by full or partial key values"
		}

		+button NextBut at 490, 80 {
			label = "&Next"
			width = 60
			event = NextButEvent
			tooltip = "Display the next record in ascending key sequence"
		}

		+button PrevBut at 490, 105 {
			label = "&Prev"
			width = 60
			event = PrevButEvent
			tooltip = "Display the previous record in descending key sequence"
		}

		+button MatchBut at 490, 130 {
			label = "&Match"
			width = 60
			event = MatchButEvent
			tooltip = "Display the next record matching the values supplied for the last Find"
		}

		+button ReportBut at 490, 155 {
			label = "&Report"
			width = 60
			event = ReportButEvent
			tooltip = "Print a report on all records in columnar form, or on the selected record"
		}

		+button ListBut at 490, 180 {
			label = "&List"
			width = 60
			event = ListButEvent
			tooltip = "Display all records in table form"
		}

		+button InsertBut at 560, 55 {
			label = "&Insert"
			width = 60
			event = InsertButEvent
			tooltip = "Insert new records"
		}

		+button AmendBut at 560, 80 {
			label = "&Amend"
			width = 60
			event = AmendButEvent
			tooltip = "Amend the currently displayed record"
		}

		+button DeleteBut at 560, 105 {
			label = "&Delete"
			width = 60
			event = DeleteButEvent
			tooltip = "Delete the currently displayed record"
		}

		+button RewindBut at 560, 130 {
			label = "&Rewind"
			width = 60
			event = RewindButEvent
			tooltip = "Rewind the file (or current secondary index) to the start"
		}

		+button WindBut at 560, 155 {
			label = "&Wind"
			width = 60
			event = WindButEvent
			tooltip = "Wind the file (or current secondary index) to the end"
		}

		+button ExitBut at 560, 180 {
			label = "&Exit"
			width = 60
			event = ExitButEvent
			tooltip = "Exit program"
		}
	}

	button group ConfirmButs {
		style = WS_DISABLED


		+button OKBut at 490, 217 {
			label = "&OK"
			width = 60
			event = OKButEvent
			style = WS_DEFAULT_BUTTON
			tooltip = "Confirm the currently displayed data"
		}

		+button CancelBut at 560, 217 {
			label = "&Cancel"
			width = 60
			event = CancelButEvent
			style = WS_CANCEL_BUTTON
			tooltip = "Cancel the current option"
		}
	}

	textbox group KeyFields {
		style = WS_NOTABSTOP

		+textbox ItemCodeBox at 78, 35 {
			field = TMP_STOCK.ItemCode
			label = "Item Code"
			width = 132
		}
	}

	textbox group DataFields {
		style = WS_NOTABSTOP

		+textbox StockLevelBox at 78, 65 {
			field = TMP_STOCK.StockLevel
			label = "Stock Level"
			width = 66
			format = "#,###,##0"
		}

		+textbox ReorderLevelBox at 78, 85 {
			field = TMP_STOCK.ReorderLevel
			label = "Reorder Level"
			width = 66
			format = "#,###,##0"
		}

		+textbox EOQBox at 78, 105 {
			field = TMP_STOCK.EOQ
			label = "EOQ"
			width = 66
			caption = "Economic order quantity for purchases"
			tooltip = "Economic order quantity for purchases"
			format = "#,###,##0"
		}

		+textbox MOQBox at 78, 125 {
			field = TMP_STOCK.MOQ
			label = "MOQ"
			width = 66
			caption = "Minimum order quantity for sales"
			tooltip = "Minimum order quantity for sales"
			format = "#,###,##0"
		}

		+textbox CostPriceBox at 78, 145 {
			field = TMP_STOCK.CostPrice
			label = "Cost Price"
			width = 72
			format = "###,###.##"
		}

		+textbox SalePriceBox at 78, 165 {
			field = TMP_STOCK.SalePrice
			label = "Sale Price"
			width = 72
			caption = "Sale price for minimum order quantity"
			tooltip = "Sale price for minimum order quantity"
			format = "###,###.##"
		}

		+textbox UnitCodeBox at 78, 185 {
			field = TMP_STOCK.UnitCode
			label = "Unit Code"
			width = 39
		}

		+textbox LeadTimeBox at 78, 205 {
			field = TMP_STOCK.LeadTime
			label = "Lead Time"
			width = 36
			caption = "Lead time in days for purchases"
			tooltip = "Lead time in days for purchases"
			format = "####"
		}

		+textbox NextDelDueBox at 78, 225 {
			field = TMP_STOCK.NextDelDue
			label = "Next Delivery"
			width = 72
			format = "dD/mM/YYYY"
		}

		+textbox DescriptionBox at 78, 245 {
			field = TMP_STOCK.Description
			label = "Description"
			width = 372
		}
	}
}

-window winIndex1 at 700, 370 {
	max_width = 200
	max_height = 120
	title = "Find by Index next_del_index"
	parent = wintask
	style = WS_OWNED
	data_font = "arial@8"
	label_font = "arial@8"


	+textbox NextDelDueBox11 at 74, 20 {
		field = TMP_STOCK.NextDelDue
		label = "Next Delivery"
		width = 72
		format = "dD/mM/YYYY"
	}

	+button Index1OKBut at 25, 60 {
		label = "&OK"
		width = 60
		event = OKFIButEvent
		style = WS_DEFAULT_BUTTON
		tooltip = "Confirm values for Find operation and close window"
	}

	+button Index1CancelBut at 100, 60 {
		label = "&Cancel"
		width = 60
		event = CancelFIButEvent
		style = WS_CANCEL_BUTTON
		tooltip = "Cancel the find operation"
	}
}


-window winList at 231, 205 {
	max_width = 1138
	max_height = 450
	parent = wintask
	style = WS_OWNED
	title = "List of records"

	+table ListTable at 8, 20 {
		source_object = FILE_STOCK
		display_object = DispListTable
		rows = 20
		cell_properties = ListTableCellProps
		event = ListTableEvent
		width = 1113
	}

	+button OKListBut at 494, 390 {
		label = "&OK"
		width = 60
		event = OKListButEvent
		style = WS_DEFAULT_BUTTON
		tooltip = "Close window, displaying selected record (if any)"
	}

	+button CancelListBut at 569, 390 {
		label = "&Cancel"
		width = 60
		event = CancelListButEvent
		style = WS_CANCEL_BUTTON
		tooltip = "Close window without displaying a record"
	}
}

-window winReport at 650, 337 {
	max_width = 300
	max_height = 185
	parent = wintask
	style = WS_OWNED
	title = "Reports"
	+graphic Graphic1 at 24,12 {
		type = GT_RECTANGLE
		width = 245
		height = 76
		title = "Select report type"
	}

	radio group RepButs {

		+button RepAllBut at 93,32 {
			label = "&All records"
			tooltip = "Print a report of all records in the file, 1 record to a line"
		}

		+button RepCurrentBut at 93,52 {
			label = "&Displayed record"
			tooltip = "Print a report of all fields in the currently selected record"
		}
	}


	+button OKRepBut at 75, 125 {
		label = "&OK"
		width = 60
		event = OKRepButEvent
		style = WS_DEFAULT_BUTTON
		tooltip = "Print selected report"
	}

	+button CancelRepBut at 150, 125 {
		label = "&Cancel"
		width = 60
		event = CancelRepButEvent
		style = WS_CANCEL_BUTTON
		tooltip = "Close window without printing a report"
	}
}

/********************************************************************************/

-window winprompt at 680, 387 {
	parent = wintask
	data_font = "arial@10"
	label_font = "arial@10"
	title = "Sculptor"
}

/********************************************************************************/

	Main()
	exit

/********************************************************************************/

!function Main() {

	Initialise()

MAIN_DIALOG:
	dialog
	return
}

/********************************************************************************/

!function Initialise() {
	openfile FILE_STOCK update
	CurrentIndex = "Main"
	SetIndexNameList()
	SetListTableCellProps()
	EnableOptButs()
	return
}

/********************************************************************************/

!function WinTaskEvent(EventCode, Object) {

	switch (EventCode) {
	case = EV_CLOSE:
		exit
	}
	return
}

/********************************************************************************/

!function Clear() {
	clear wintask

	clear FILE_STOCK
	clear TMP_STOCK
	Selected = FALSE
	return
}

/********************************************************************************/

!function Display() {

	TMP_STOCK = FILE_STOCK
	display wintask
	Selected = TRUE
	LastRec = FILE_STOCK
	sync_indexes FILE_STOCK
	return
}

/********************************************************************************/

!function SetIndexNameList() {
	NumSelIndexes = 1
	listbox_add(IndexList,"Main", 0)
	listbox_add(IndexList,"next_del_index",0)
	set_selected(IndexList, 1, TRUE)
	return
}

/********************************************************************************/

!function IndexListEvent(EventCode, Object) {
	if (EventCode <> EV_SINGLE_CLICK) then return
	listbox_get_selected(IndexList, CurrentIndex, CurrentIndexNum)
	if (CurrentIndex <> LastFindIndex) or (Found = FALSE) then disable MatchBut else enable MatchBut
	return
}

/********************************************************************************/

!function FindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	Clear()
	CurrentOpt = _FIND
	if (CurrentIndex = "Main") then FindMain()

	if (CurrentIndex = "next_del_index") {
		FindIndex(winIndex1)
	}
	return
}

/********************************************************************************/

!function FindMain() {

	disable OptButs
	enable ConfirmButs
	enable_tabstop KeyFields
	disable_tabstop IndexList
	SetFocusKey1()
	return
}

/********************************************************************************/

!function FindIndex(WinName) {

	clear WinName
	show WinName
	dialog WinName
	hide WinName
	if (ContinueFind = TRUE) then OKFind()
	return
}

/********************************************************************************/

!function OKFIButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	ContinueFind = TRUE
	return CANCEL_DIALOG
}

/********************************************************************************/

!function CancelFIButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	ContinueFind = FALSE
	return CANCEL_DIALOG
}

/********************************************************************************/

!function OKFind() {

	FILE_STOCK = TMP_STOCK
	if (CurrentIndex = "Main") {
		findu FILE_STOCK nsr = FIND_NSR riu = FIND_RIU traps = FIND_TRAPS
	}

	if (CurrentIndex = "next_del_index") {
		findu FILE_STOCK index = next_del_index nsr = FIND_NSR riu = FIND_RIU traps = FIND_TRAPS
	}

	Found = TRUE
	Display()
	DisableInput()
	LastFindIndex = CurrentIndex
	setfocus FindBut
	return

FIND_NSR:
	info "No record found"
	SetFocusKey1()
	return

FIND_RIU:
	info "Record is locked by another user"
	SetFocusKey1()
	return

FIND_TRAPS:
	info "Cannot find record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function NextButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (CurrentIndex = "Main") {
		nextu FILE_STOCK nsr = NEXT_NSR riu = NEXT_RIU traps = NEXT_TRAPS
	}

	if (CurrentIndex = "next_del_index") {
		nextu FILE_STOCK index = next_del_index nsr = NEXT_NSR riu = NEXT_RIU traps = NEXT_TRAPS
	}

	Display()
	EnableOptButs()
	return

NEXT_NSR:
	info "End of file"
	return

NEXT_RIU:
	info "Next record is locked by another user"
	return

NEXT_TRAPS:
	info "Cannot find next record"
	return
}

/********************************************************************************/

!function PrevButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (CurrentIndex = "Main") {
		prevu FILE_STOCK nsr = PREV_NSR riu = PREV_RIU traps = PREV_TRAPS
	}

	if (CurrentIndex = "next_del_index") {
		prevu FILE_STOCK index = next_del_index nsr = PREV_NSR riu = PREV_RIU traps = PREV_TRAPS
	}

	Display()
	EnableOptButs()
	return

PREV_NSR:
	info "Beginning of file"
	return

PREV_RIU:
	info "Previous record is locked by another user"
	return

PREV_TRAPS:
	info "Cannot find previous record"
	return
}

/********************************************************************************/

!function MatchButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (CurrentIndex = "Main") {
		matchu FILE_STOCK nsr = MATCH_NSR riu = MATCH_RIU traps = MATCH_TRAPS
	}

	if (CurrentIndex = "next_del_index") {
		matchu FILE_STOCK index = next_del_index nsr = MATCH_NSR riu = MATCH_RIU traps = MATCH_TRAPS
	}

	Display()
	EnableOptButs()
	return

MATCH_NSR:
	info "No more matching records"
	return

MATCH_RIU:
	info "Next matching record is locked by another user"
	return

MATCH_TRAPS:
	info "Cannot find matching record"
	return
}

/********************************************************************************/

!function ReportButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	show winReport
	switch (Selected) {
		case = FALSE:
			disable RepCurrentBut
			RepAllBut->checked = TRUE
			break

		case = TRUE:
			enable RepCurrentBut
			break
	}

	dialog winReport cancel = RE_CANCEL

RE_CANCEL:
	hide winReport
	return
}

/********************************************************************************/

!function OKRepButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (RepCurrentBut->checked = TRUE) {
		run report PrintCurrent to "printmanager" ppf = "winpm"
	}

	if (RepAllBut->checked = TRUE) {
		if (CurrentIndex = "Main") {
			run report PrintAll to "printmanager" ppf = "winpm"
		}

		if (CurrentIndex = "next_del_index") {
			run report PrintAll1 to "printmanager" ppf = "winpm"
		}
	}

	return CANCEL_DIALOG
}

/********************************************************************************/

!function CancelRepButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	return CANCEL_DIALOG
}

/********************************************************************************/

!function ListButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	destroy ListTable
	if (CurrentIndex = "Main") {
		ListTable->source_object = FILE_STOCK
	}

	if (CurrentIndex = "next_del_index") {
		ListTable->source_object = FILE_STOCK.next_del_index
	}

	winList->title = "List of records sorted by " + CurrentIndex / " index"
	create ListTable
	set_selected(ListTable, 0, FALSE)
	display ListTable

	show winList
	dialog winList
	hide winList
	return
}

/********************************************************************************/

!function DisplayListRec() {
	if (table_first_selected_key(ListTable) = OKAY) {
		if (ReadRecord(_UNLOCK, "Main") = OKAY) {
			sync_indexes FILE_STOCK
			Display()
			EnableOptButs()
		}
	}
	return
}

/********************************************************************************/

!function OKListButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	DisplayListRec()
	return CANCEL_DIALOG
}

/********************************************************************************/

!function CancelListButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	return CANCEL_DIALOG
}

/********************************************************************************/

!function SetListTableCellProps() {

	ListTableCellProps.cell_type[1] = TCT_READONLY
	ListTableCellProps.cell_type[2] = TCT_READONLY
	ListTableCellProps.cell_type[3] = TCT_READONLY
	ListTableCellProps.cell_type[4] = TCT_READONLY
	ListTableCellProps.cell_type[5] = TCT_READONLY
	ListTableCellProps.cell_type[6] = TCT_READONLY
	ListTableCellProps.cell_type[7] = TCT_READONLY
	ListTableCellProps.cell_type[8] = TCT_READONLY
	ListTableCellProps.cell_type[9] = TCT_READONLY
	ListTableCellProps.cell_type[10] = TCT_READONLY
	ListTableCellProps.cell_type[11] = TCT_READONLY
	create ListTable
	return
}

/********************************************************************************/

!function ListTableEvent(EventCode, Table, Line) {

	if (EventCode = EV_DOUBLE_CLICK) {
		DisplayListRec()
		return CANCEL_DIALOG
	}
	return
}

/********************************************************************************/

!function InsertButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	Clear()
	clear LastRec
	Inserted = FALSE
	CurrentOpt = _INSERT
	EnableInput()
	SetFocusKey1()
	return
}

/********************************************************************************/

!function OKInsert() {

	if (isblank(TMP_STOCK)) {
		prompt "", \
			" Insert a record with all null values - are you sure? " \
				button = " Yes " \
				button = " No  ", INSERT_CONTINUE
	}

	CopyToFile()
	insert FILE_STOCK re = INSERT_RE riu = INSERT_RIU traps = INSERT_TRAPS

	Selected = TRUE
	Inserted = TRUE
	LastRec = FILE_STOCK
	Clear()
	EnableInput()
	SetFocusKey1()
	return

INSERT_RE:
	info "Record already exists with this key"

INSERT_CONTINUE:
	SetFocusKey1()
	return

INSERT_RIU:
	info "File has been locked by another process"
	SetFocusKey1()
	return

INSERT_TRAPS:
	info "Cannot insert record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function AmendButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	if (ReadRecord(_LOCK, "Main") = ERROR) then return

	CurrentOpt = _AMEND
	EnableInput()

	SetFocusKey1()
	return
}

/********************************************************************************/

!function OKAmend() {

	CopyToFile()
	write FILE_STOCK re = AMEND_RE nrs = AMEND_NRS ue = AMEND_UE  traps = AMEND_TRAPS
	Selected = TRUE
	DisableInput()
	setfocus AmendBut
	return

AMEND_RE:
	info "Record already exists with this key"
	SetFocusKey1()
	return

AMEND_NRS:
	info "No record selected"
	Display()
	DisableInput()
	setfocus AmendBut
	return

AMEND_UE:
	info "Update refused - record updated by another user"
	Display()
	DisableInput()
	setfocus AmendBut
	return

AMEND_TRAPS:
	info "Cannot amend record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function DeleteButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	if (ReadRecord(_LOCK, "Main") = ERROR) then return

	prompt "", \
		" Delete this record? " \
			button = " Yes " \
			button = " No  ", DEL_RETURN

	delete FILE_STOCK nrs = DEL_NRS traps = DEL_TRAPS
	Clear()
	EnableOptButs()
	return

DEL_NRS:
	info "No record selected"
	return

DEL_TRAPS:
	info "Unable to delete record"
	return

DEL_RETURN:
	unlock FILE_STOCK
	return
}

/********************************************************************************/

!function RewindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (CurrentIndex = "Main") {
		rewind FILE_STOCK
	}

	if (CurrentIndex = "next_del_index") {
		rewind FILE_STOCK index = next_del_index
	}

	Clear()

	switch(CurrentIndex) {
		case = "Main":
			caption "File has been rewound to the beginning"
			break

		default:
			caption "Index " + CurrentIndex / " has been rewound to the beginning"
			break
	}

	return
}

/********************************************************************************/

!function WindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (CurrentIndex = "Main") {
		wind FILE_STOCK
	}

	if (CurrentIndex = "next_del_index") {
		wind FILE_STOCK index = next_del_index
	}

	Clear()

	switch(CurrentIndex) {
		case = "Main":
			caption "File has been wound to the end"
			break

		default:
			caption "Index " + CurrentIndex / " has been wound to the end"
			break
	}

	return
}

/********************************************************************************/

!function ExitButEvent(EventCode, Object) {
	if (EventCode = EV_BUTTON_CLICKED) then exit
	return
}

/********************************************************************************/

!function OKButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	switch (CurrentOpt) {
		case = _FIND:
			OKFind()
			break

		case = _INSERT:
			OKInsert()
			break

		case = _AMEND:
			OKAmend()
			break
	}
	return
}

/********************************************************************************/

!function CancelButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	switch (CurrentOpt) {
		case = _FIND:
			Clear()
			setfocus FindBut
			break

		case = _INSERT:
			DisplayLastRecord()
			setfocus InsertBut
			break

		case = _AMEND:
			unlock FILE_STOCK
			Display()
			setfocus AmendBut
			break
	}

	DisableInput()
	return
}

/********************************************************************************/

!function EnableOptButs() {
	enable OptButs
	if (Found = FALSE) then disable MatchBut
	if (Selected = FALSE) then disable AmendBut, DeleteBut
	return
}

/********************************************************************************/

!function EnableInput() {
	disable_tabstop IndexList
	enable_tabstop KeyFields, DataFields
	disable OptButs
	enable ConfirmButs
	return
}

/********************************************************************************/

!function DisableInput() {
	enable_tabstop IndexList
	disable_tabstop KeyFields, DataFields
	disable ConfirmButs
	EnableOptButs()
	return
}

/********************************************************************************/

!function ReadRecord(Mode, Index) {

	if (Index = "") then Index = CurrentIndex
	if (Mode = _LOCK) {

		if (Index = "Main") {
			read FILE_STOCK nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
		}

		if (Index = "next_del_index") {
			read FILE_STOCK index = next_del_index nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
		}
	}

	if (Mode <> _LOCK) {

		if (Index = "Main") {
			readu FILE_STOCK nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
		}

		if (Index = "next_del_index") {
			readu FILE_STOCK index = next_del_index nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
		}
	}

	return OKAY

READ_NSR:
	info "Record has been deleted by another user"
	Clear()
	return ERROR

READ_RIU:
	info "Record is locked by another user"
	return ERROR

READ_TRAPS:
	info "Cannot reread record (unknown error)"
	return ERROR

}

/********************************************************************************/

!function CopyToFile() {
	FILE_STOCK = TMP_STOCK
	return
}

/********************************************************************************/

!function DisplayLastRecord() {

	if (Inserted = FALSE) then goto DLR_CLEAR
	FILE_STOCK = LastRec
	read FILE_STOCK traps = DLR_CLEAR
	Display()
	return

DLR_CLEAR:
	Clear()
	return
}

/********************************************************************************/

!function SetFocusKey1() {
	setfocus ItemCodeBox
	return
}

/********************************************************************************/

!report PrintAll {

!drive FILE_STOCK

!heading {
	PrintTitle()
	PrintHeadings()
}

!final {
	PrintFinal()
}

	PrintRecord()
	end

}

/********************************************************************************/

!report PrintAll1 {

!drive FILE_STOCK index = next_del_index

!heading {
	PrintTitle()
	print "Report by index next_del_index"
	PrintHeadings()
}

!final {
	PrintFinal()
}

	PrintRecord()
	end

}

/********************************************************************************/

!report PrintCurrent {

!temp DataCol,,i2
!temp Element,,i2
!temp PrintHD,,a255

!heading {
	PrintTitle()
}

!title {
	DataCol = 20
	sys.LeftMargin = 10
	sys.LineHeight = sys.CharHeight
	PrintTitle()
	print fit("Item Code:", 19); tab(DataCol); FILE_STOCK.ItemCode
	print fit("Stock Level:", 19); tab(DataCol); FILE_STOCK.StockLevel
	print fit("Reorder Level:", 19); tab(DataCol); FILE_STOCK.ReorderLevel
	print fit("EOQ:", 19); tab(DataCol); FILE_STOCK.EOQ
	print fit("MOQ:", 19); tab(DataCol); FILE_STOCK.MOQ
	print fit("Cost Price:", 19); tab(DataCol); FILE_STOCK.CostPrice
	print fit("Sale Price:", 19); tab(DataCol); FILE_STOCK.SalePrice
	print fit("Unit Code:", 19); tab(DataCol); FILE_STOCK.UnitCode
	print fit("Lead Time:", 19); tab(DataCol); FILE_STOCK.LeadTime
	print fit("Next Delivery:", 19); tab(DataCol); FILE_STOCK.NextDelDue
	print fit("Description:", 19); tab(DataCol); FILE_STOCK.Description
 	end report
}

	end

}

/********************************************************************************/

!function PrintTitle() {

	print lns(2)
	setfont("Arial", 10, FONT_BOLD)
	print "stock", sys.Date
	setfont(sys.FontName, sys.FontSize, FONT_NORMAL)
	print
	return
}

/********************************************************************************/

!function PrintHeadings() {

	print
	setfont(sys.FontName, sys.FontSize, FONT_UNDERLINE)
	print tab(1); fit("Item Code", 20);
	print tab(23); fit("Stock Level", 9);
	print tab(45); fit("Reorder Level", 9);
	print tab(67); fit("EOQ", 9);
	print tab(89); fit("MOQ", 9);
	print tab(111); fit("Cost Price", 10);
	print tab(133); fit("Sale Price", 10);
	print tab(155); fit("Unit Code", 5);
	print tab(177); fit("Lead Time", 5);
	print tab(199); fit("Next Delivery", 10);
	print tab(221); fit("Description", 60);
	print
	setfont(sys.FontName, sys.FontSize, FONT_NORMAL)
	print
	return
}

/********************************************************************************/

!function PrintFinal() {

	print
	print sys.RecordCount: :"s"; " records printed"
	if (Selected = TRUE) {
	FILE_STOCK = TMP_STOCK
		ReadRecord(_UNLOCK, "Main")
	}
	return
}

/********************************************************************************/

!function PrintRecord() {
	print tab(1); fit(FILE_STOCK.ItemCode, 20);
	print tab(23); fit(FILE_STOCK.StockLevel, 9) : "#,###,##0";
	print tab(45); fit(FILE_STOCK.ReorderLevel, 9) : "#,###,##0";
	print tab(67); fit(FILE_STOCK.EOQ, 9) : "#,###,##0";
	print tab(89); fit(FILE_STOCK.MOQ, 9) : "#,###,##0";
	print tab(111); fit(FILE_STOCK.CostPrice, 10) : "###,###.##";
	print tab(133); fit(FILE_STOCK.SalePrice, 10) : "###,###.##";
	print tab(155); fit(FILE_STOCK.UnitCode, 5);
	print tab(177); fit(FILE_STOCK.LeadTime, 5) : "####";
	print tab(199); fit(FILE_STOCK.NextDelDue, 10) : "dD/mM/YYYY";
	print tab(221); fit(FILE_STOCK.Description, 60)
	return
}

/********************************************************************************/
!chargrid OFF
/********************************************************************************/
